#!/bin/bash

# Path to the original JSX script
ORIGINAL_JSX_SCRIPT="bin/export_pdf.jsx"  # Update with the correct path

# Path to the system's temporary directory
TEMP_DIR=$(mktemp -d)

# Path to the modified JSX script
MODIFIED_JSX_SCRIPT="$TEMP_DIR/export_png_modified.jsx"

# Check if the original JSX script exists
if [ ! -f "$ORIGINAL_JSX_SCRIPT" ]; then
    echo "Error: Original JSX script not found: $ORIGINAL_JSX_SCRIPT"
    exit 1
fi

# Check if an argument is provided
if [ -z "$1" ]; then
    echo "Error: No argument provided. Please provide the full name of the INDD file."
    exit 1
fi

# Create the modified JSX script by copying the original
cp "$ORIGINAL_JSX_SCRIPT" "$MODIFIED_JSX_SCRIPT"

# Replace the folder path in the modified JSX script
sed -i '' "s|var inddFilePath = \".*\";|var inddFilePath = \"$1\";|g" "$MODIFIED_JSX_SCRIPT"

# Execute the JSX script in Adobe InDesign
osascript -e "tell application id \"com.adobe.indesign\" to do script (posix file \"$MODIFIED_JSX_SCRIPT\") language javascript"

# Clean up the temporary directory
rm -rf "$TEMP_DIR"