from os import path

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.colors import to_rgba
from snvc.data_handling import REPO_PATH

plt.style.use(path.join(REPO_PATH, "src", "snvc", "paper.mplstyle"))

# Figure Sizes
MM = 1 / 25.4
COL = 86 * MM
COL_DUB = 178 * MM

ALPHA_TRANSPARENT = 0.7
FIG_MARGIN = 1.5

DATA_COLOR = "#253b6e"
DATA_COLOR_TRANSPARENT = to_rgba(DATA_COLOR, ALPHA_TRANSPARENT)
DATA_KW = {"markeredgecolor": DATA_COLOR, "markerfacecolor": DATA_COLOR_TRANSPARENT}
DATA_SCATTER_KW = {"edgecolor": DATA_COLOR, "facecolor": DATA_COLOR_TRANSPARENT}
OPTIMAL_KW = {"color": "black", "linestyle": "dashed"}
FIT_KW = {"color": DATA_COLOR_TRANSPARENT, "linestyle": "-", "zorder": 100}
COHERENCE_KW = {"linestyle": "-.", "color": "gray", "alpha": 0.5}
TRANSITION_DOWN_COLOR = "#9c1111"
TRANSITION_UP_COLOR = "#f7543b"
GREEN = "#1d7b3d"
GREEN_BLUE = "#00777a"
YELLOW = "#f1b71c"


def set_margins(fig, margin_mm=FIG_MARGIN):
    w = fig.get_figwidth() / MM
    h = fig.get_figheight() / MM
    fig.get_layout_engine().set(
        rect=[
            margin_mm / w,
            margin_mm / h,
            (w - 2 * margin_mm) / w,
            (h - 2 * margin_mm) / h,
        ]
    )
    fig.get_layout_engine().execute(fig)
    return fig


def p0_to_sigmaz(ds, p0_name="p0"):
    ds["sigmaz"] = 2 * getattr(ds, p0_name) - 1
    ds.sigmaz.attrs["long_name"] = r"$\langle \sigma_z \rangle$"
    try:
        ds["sigmaz_u"] = 2 * getattr(ds, f"{p0_name}_u")
        ds.sigmaz_u.attrs["long_name"] = r"$\langle \sigma_z^u \rangle$"
    except:
        pass
    return ds
