import numpy as np


def time_cpmg(N, total_time):
    tau = total_time / N / 2
    i = np.arange(N + 2)
    t = 2 * tau * (i - 0.5)
    t[0] = 0
    t[-1] = total_time
    return t


def time_delta_cpmg(N, total_time):
    tau = total_time / N / 2
    t_delta = np.full(N + 1, 2 * tau)
    t_delta[0] = tau
    t_delta[-1] = tau
    return t_delta


def time_udd(N, total_time):
    i = np.arange(N + 2)
    t = total_time * np.sin(i * np.pi / (2 * N + 2)) ** 2
    return t


def time_delta_udd(N, total_time):
    t = time_udd(N=N, total_time=total_time)
    t_delta = t[1:] - t[:-1]
    return t_delta


def time(N, total_time, dd):
    if dd == "cpmg":
        return time_cpmg(N=N, total_time=total_time)
    elif dd == "udd":
        return time_udd(N=N, total_time=total_time)
    else:
        raise NotImplementedError


def time_delta(N, total_time, dd):
    if dd == "cpmg":
        return time_delta_cpmg(N=N, total_time=total_time)
    elif dd == "udd":
        return time_delta_udd(N=N, total_time=total_time)
    else:
        raise NotImplementedError


def phase(time_deltas, f0, f1, phase_difference=np.pi):
    phase0_array = np.zeros(len(time_deltas) + 1)
    phase1_array = np.zeros(len(time_deltas) + 1)
    rf_phase0_array = np.zeros(len(time_deltas) + 1)
    rf_phase1_array = np.zeros(len(time_deltas) + 1)

    phase0 = phase1 = 0

    for i, delta in enumerate(time_deltas):
        spin = i % 2

        phase0_array[i] = phase0
        phase1_array[i] = phase1

        if spin == 0:
            rf_phase0_array[i] = phase0
            rf_phase1_array[i] = phase1 + phase_difference
        else:
            rf_phase0_array[i] = phase1 + phase_difference
            rf_phase1_array[i] = phase0

        if spin == 0:
            phase0 += f0 * delta * 2 * np.pi
            phase1 += f1 * delta * 2 * np.pi
        else:
            phase0 += f1 * delta * 2 * np.pi
            phase1 += f0 * delta * 2 * np.pi

    phase0_array[-1] = phase0
    phase1_array[-1] = phase1
    if spin == 1:
        rf_phase0_array[-1] = phase0
        rf_phase1_array[-1] = phase1 + phase_difference
    else:
        rf_phase0_array[-1] = phase1 + phase_difference
        rf_phase1_array[-1] = phase0

    return phase0_array, phase1_array, rf_phase0_array, rf_phase1_array


def phase_dd(N, total_time, dd, f0, f1, phase_difference=np.pi, return_degrees=False):
    dd_dict = {}
    dd_dict["time"] = time(N, total_time, dd)
    dd_dict["time_delta"] = time_delta(N, total_time, dd)
    phase0_array, phase1_array, rf_phase0_array, rf_phase1_array = phase(
        dd_dict["time_delta"], f0, f1, phase_difference
    )
    dd_dict["phase0"] = phase0_array
    dd_dict["phase1"] = phase1_array
    dd_dict["rf_phase0"] = rf_phase0_array
    dd_dict["rf_phase1"] = rf_phase1_array
    if return_degrees:
        dd_dict["phase0"] *= 180 / np.pi
        dd_dict["phase1"] *= 180 / np.pi
        dd_dict["rf_phase0"] *= 180 / np.pi
        dd_dict["rf_phase1"] *= 180 / np.pi
    return dd_dict


def phase_dd_spin(
    N, total_time, dd, coupling, phase_difference=np.pi, spin=0.5, return_degrees=False
):
    if spin == 0.5:
        f0 = -coupling / 2
        f1 = coupling / 2
    elif spin == 1:
        f0 = 0
        f1 = coupling
    else:
        raise NotImplementedError
    return phase_dd(
        N,
        total_time,
        dd,
        f0,
        f1,
        phase_difference=phase_difference,
        return_degrees=return_degrees,
    )
